/*
 * Zynaptic Stash - An asynchronous persistence framework for Java.
 * 
 * Copyright (c) 2009-2015, Zynaptic Limited. All rights reserved. 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 only, as published by
 * the Free Software Foundation. Zynaptic Limited designates this particular
 * file as subject to the "Classpath" exception as provided in the LICENSE
 * file that accompanied this code.
 * 
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 * 
 * You should have received a copy of the GNU General Public License version 2
 * along with this work; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * Please visit www.zynaptic.com or contact reaction@zynaptic.com if you need
 * additional information or have any questions.
 */

package com.zynaptic.stash.core.file;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;

/**
 * Subclass the input object data stream to force it to use a specific class
 * loader. This allows hidden classes in other OSGi bundles to be correctly
 * handled by the stash service.
 * 
 * @author Chris Holgate
 */
public final class ResolvedObjectInputStream extends ObjectInputStream {
  private final ClassLoader dataClassLoader;

  /**
   * Provides the default constructor for object input streams with specific
   * data class loader object resolution.
   * 
   * @param in
   *          This is the input stream to read from.
   * @param dataClassLoader
   *          This is the class loader which should be used to resolve the data
   *          classes in the input stream.
   * @throws IOException
   *           This exception is thrown if an I/O error occurs while reading the
   *           wrapped input stream.
   */
  public ResolvedObjectInputStream(InputStream in, ClassLoader dataClassLoader)
      throws IOException {
    super(in);
    this.dataClassLoader = dataClassLoader;
  }

  /*
   * Overrides the default class resolution to use the supplied data class
   * loader.
   */
  @Override
  protected Class<?> resolveClass(ObjectStreamClass desc) throws IOException,
      ClassNotFoundException {
    return Class.forName(desc.getName(), false, dataClassLoader);
  }
}
